import React, { useState, useRef, useEffect } from "react";
import ReactCrop, { centerCrop, makeAspectCrop, convertToPixelCrop } from "react-image-crop";
import "react-image-crop/dist/ReactCrop.css";
import { canvasPreview } from "./canvasPreview";
import { getStoredValue, saveToLocalStorage } from "@/app/handlers/localStorageHandler";
import mecredApi from "@/axiosConfig";

export default function ImageCropper({ payloadHeader, type, userId, setChangePhoto }) {
    // Estados para gerenciar a foto original, URL da foto, configuração do corte, corte completo e imagem cortada
    const [photo, setPhoto] = useState(null);
    const [photoURL, setPhotoURL] = useState(null);
    const [crop, setCrop] = useState(null);
    const [completedCrop, setCompletedCrop] = useState(false);
    const [croppedImage, setCroppedImage] = useState(null);

    // Referências para a imagem e o canvas de pré-visualização
    const imgRef = useRef(null);
    const previewCanvasRef = useRef(null);

    // Recupera tokens de autenticação do armazenamento local
    const token = getStoredValue("access_token");
    const client = getStoredValue("client");
    const uid = getStoredValue("uid");

    // Hook personalizado para aplicar debounce em efeitos
    function useDebounceEffect(fn, waitTime, deps = []) {
        useEffect(() => {
            const t = setTimeout(() => {
                fn.apply(undefined, deps);
            }, waitTime);
        
            return () => {
                clearTimeout(t);
            };
        }, deps);
    }

    // Função para fazer upload da foto cortada
    const uploadPhoto = async () => {
        let payload = new FormData();
        payload.set(payloadHeader, croppedImage);
        
        await mecredApi.put(`/${type}/${userId}`, payload, {
            headers: {
                'access-token': token,
                'token-type': 'Bearer',
                'client': client,
                'uid': uid,
                'Expires': 0
            }
        })
        .catch(error => {
            setChangePhoto(false);
            if (error?.response.status === 500) // TODO: alterar isso quando tivermos um novo backend, pois não deveria retornar 500 toda vez.
                console.error(error); // Talvez o backend tenha falhado, mas provavelmente a foto do perfil foi alterada.
        });
        
        await mecredApi.get(`/users/${userId}`, {
            headers: {
                'access-token': token,
                'token-type': 'Bearer',
                'client': client,
                'uid': uid,
                'Expires': 0
            }
        })
        .then(res => {
            let userData = JSON.parse(getStoredValue("user_data"));
            userData["avatar_file_name"] = res.data.avatar;
            saveToLocalStorage("user_data", JSON.stringify(userData));
        });
    };
      
    // Manipulador para quando o usuário seleciona uma foto
    const handlePhoto = (e) => {
        e.preventDefault();
        const file = e.target.files[0];
        if (file) {
            setPhoto(file);
            setPhotoURL(URL.createObjectURL(file));
        }
    };

    // Função para centralizar o corte com uma proporção específica
    function centerAspectCrop(mediaWidth, mediaHeight, aspect) {
        return centerCrop(
            makeAspectCrop(
                { unit: "%", width: 50 },
                aspect,
                mediaWidth,
                mediaHeight
            ),
            mediaWidth,
            mediaHeight
        );
    }

    // Função chamada quando a imagem é carregada
    function onImageLoad(e) {
        const { width, height } = e.currentTarget;
        const newCrop = centerAspectCrop(width, height, 1);
        setCrop(newCrop);
        setCompletedCrop(convertToPixelCrop(newCrop, width, height));
    }

    // Efeito com debounce para gerar a pré-visualização da imagem cortada
    useDebounceEffect(
        () => {
            if (
                completedCrop?.width &&
                completedCrop?.height &&
                imgRef.current &&
                previewCanvasRef.current
            ) {
                // Gera a pré-visualização no canvas
                canvasPreview(
                    imgRef.current,
                    previewCanvasRef.current,
                    completedCrop,
                    1,
                    0
                );
                // Converte o conteúdo do canvas em um blob e atualiza o estado
                previewCanvasRef.current.toBlob((blob) => {
                    console.log("meu blob: ", blob);
                    setCroppedImage(blob);
                }); 
            }
        },
        100,
        [completedCrop]
    );

    return (
        <div className="flex flex-col items-center">
            <div className="flex space-x-4 mt-2 mb-5">
                <label className="bg-turquoise text-white-HC-dark-underline rounded-lg p-2 cursor-pointer">
                    <input type="file" onChange={handlePhoto} className="hidden" />
                    Selecionar Nova Foto
                </label>
                {completedCrop && (
                    <button
                        className="text-sm p-2 text-white-HC-dark-underline border-main rounded-lg font-bold bg-turquoise hover:bg-turquoise-hover"
                        onClick={uploadPhoto}
                    >
                        Enviar
                    </button>
                )}
            </div>
            {photoURL && (
                <div className="flex space-x-4">
                    <ReactCrop
                        crop={crop}
                        onChange={(_, percentCrop) => setCrop(percentCrop)}
                        onComplete={(c) => setCompletedCrop(c)}
                        aspect={1}
                        circularCrop={true}
                    >
                        <img
                            ref={imgRef}
                            alt="Imagem de Perfil"
                            src={photoURL}
                            onLoad={onImageLoad}
                            style={{ maxWidth: '400px', maxHeight: '400px' }}
                        />
                    </ReactCrop>
                    {completedCrop && (
                        <div className="flex justify-center items-center">
                            <canvas
                                ref={previewCanvasRef}
                                style={{
                                    borderRadius: "50%",
                                    border: "2px solid black",
                                    objectFit: "contain",
                                    width: Math.min(completedCrop.width, 200),
                                    height: Math.min(completedCrop.height, 200),
                                }}
                            />
                        </div>
                    )}
                </div>
            )}
        </div>
    );
}
