import React, { useState, useEffect, useCallback } from "react";
import InfiniteScrollCards from "./InfiniteScrollCards";''
import mecredApi from "@/axiosConfig";
import Loading from "./Loading";
import { useSearchParams, useRouter } from "next/navigation";

const getUrlFromFilterState = (filterState, page) => {
  const apiParams = new URLSearchParams();
  apiParams.append("page", page);
  apiParams.append("results_per_page", "20");
  apiParams.append("order", filterState.order);
  apiParams.append("query", filterState.query);
  apiParams.append("search_class", filterState.searchClass);
  apiParams.append("subjects[]", filterState.subjects.map((obj) => obj.id).toString());
  apiParams.append("object_types[]", filterState.objectTypes.map((obj) => obj.id).toString());
  apiParams.append("educational_stages[]", filterState.educationalStages.map((obj) => obj.id).toString());
  apiParams.append("languages[]", filterState.languages.map((obj) => obj.id).toString());
  
  return `/search?${apiParams.toString()}`;
}

/**
 * @param {Object} props
 * @param {String} props.filterSubject matérias selecionadas - string de números inteiros, cada um correspondendo a uma máteria
 * @param {String} props.query query digitanda na barra de busca do site
 * @param {String} props.type nome em inglês da página (ex:collections)
 * @param {String} props.filter nome do filtro selecionado (ex:publicationdesc)
 * @param {Function} props.setNewSize
 * @param {Boolean} props.newSize boolean que verifica se o tamanho da tela foi alterado
 * @returns  faz a chamada dos recursos/coleções conforme a tela é scrollada ou os filtros alterados
 */
export default function InfiniteScroll({ filterState, setNewSize, newSize, setItems, items, abortController }) {
  const [isLoading, setIsLoading] = useState(false);
  const [page, setPage] = useState(0);
  const [isScroll, setIsScroll] = useState(false)
  const router = useRouter();

  // começa com 1 pq quando carrega a página a primeira vez, caso zero, a mensagem aparece por alguns segundos
  const [totalCount, setTotalCount] = useState(1)

  const fetchData = useCallback(async (page) => {
    setIsLoading(true);
    const url = getUrlFromFilterState(filterState, page);
    try {
      const { data, headers } = await mecredApi.get(url, {
        signal: abortController.signal
      });
      // verificando se a conteudo para a url passada, caso nao tenha emite mensagem de conteudo nao enconttrado
      setTotalCount(Number(headers["x-total-count"]))

      setItems((prevItems) => {
        // TODO: O backend retorna itens repetidos porque ordena as páginas por
        // um atributo que pode repetir. A solução adequada é que o backend ordene
        // por mais de um campo, por exemplo pelo filtro passado e depois pelo id,
        // assim as ordenações serão estáveis.
        // Solução hack atual: remover os itens duplicados.
        const idsSeen = new Set();
        const withoutDupes = [];
        [...page === 0 ? [] : prevItems, ...data].forEach((item) => {
          if (idsSeen.has(item.id)) { return; }
          idsSeen.add(item.id);
          withoutDupes.push(item);
        });
        return withoutDupes;
      });

      setPage(() => page + 1);
    } catch (error) {
      console.error(error);
    } finally {
      setIsLoading(false);
      setIsScroll(false)
    }
  }, [filterState, setItems]);

  //verifica se o usuário chegou no final da página
  const handleScroll = useCallback(() => {
      // Verifica se o usuário está próximo do final da área rolável
    if (
      window.innerHeight * 4 + window.scrollY < document.documentElement.offsetHeight || 
      isLoading
    ) {
      return;
    }
  
    // Busca mais dados
    fetchData(page);
  }, [fetchData, isLoading, page]);
  
  //caso o filtro seja alterado, seta o site para o topo da tela
  useEffect(() => {
    fetchData(0).then(() => { window.scrollTo(0, 0) });
    setIsScroll(true);
  }, [fetchData]);

  //controle do scroll da página
  useEffect(() => {
    window.addEventListener("scroll", handleScroll);
    return () => window.removeEventListener("scroll", handleScroll);
  }, [isLoading, handleScroll]);

  return (
    <>

      <div className={`${isScroll ? " blur-sm" : ""}`}>
        {totalCount ?

          <InfiniteScrollCards setNewSize={setNewSize} newSize={newSize} data={items} searchClass={filterState?.searchClass} />
          :
          <div className="flex justify-center text-2xl font-bold text-darkGray-HC-white mt-72 text-center">
            Desculpe, não encontramos nada.
            <br />
            {": ("}
          </div>

        }
      </div>
      {isLoading &&
        <Loading  scroll={isScroll} />
      }
    </>

  );
}
