"use client";
import Image from "next/image";
import { useState } from 'react';
import {
    Button,
    Paper,
    Alert,
    TextField
} from "@mui/material";
import theme from "@/app/theme";
import { ThemeProvider } from "@emotion/react";
import { useRouter } from "next/navigation";
import mecredApi from "@/axiosConfig";
import { removeFromLocalStorage } from "../handlers/localStorageHandler";

/**
 * Função que reseta a senha do usuário
 */
export default function ResetPassword() {

    const [password, setPassword] = useState('');
    const [passwordConfirmation, setPasswordConfirmation] = useState('');
    const [errorMessages, setErrorMessages] = useState([]);
    const [showPassword, setShowPassword] = useState(false);
    const [showPasswordConfirmation, setShowPasswordConfirmation] = useState(false);

    const router = useRouter();

    /**
     * Função para disparaa à API a nova senha do usuário.
     * Redireciona para a tela de /entrar
     * @param {Event} e - evento do onClick do botão de enviar form
     */
    const onSubmit = async (e) => {
        e.preventDefault()

        // Verifica password
        if (!password || !passwordConfirmation) {
            setErrorMessages(["Todos os campos marcados com * são obrigatórios!"]);
            return;
        }

        if (password != passwordConfirmation) {
            setErrorMessages(["As senhas devem ser iguais!"]);
            return;
        }

        // Constrói o payload de acordo com a URL
        const urlParams = new URLSearchParams(window.location.search);
        const clientId = urlParams.get("client_id");
        const config = urlParams.get("config");
        const expiry = urlParams.get("expiry");
        const resetPassword = urlParams.get("reset_password");
        const token = urlParams.get("token");
        const uid = urlParams.get("uid");
        const url = `/auth/password`

        const payload = {
            "password": e.target["senha"].value,
            "password_confirmation": e.target["confirma-senha"].value,
            "client": clientId,
            "config": config,
            "expiry": expiry,
            "reset_password": resetPassword,
            "access-token": token,
            "uid": uid
        }

        await mecredApi.put(url, payload)
            .then(res => {
                removeFromLocalStorage("access_token"); // Deleta o token de acesso, se houver
                router.push("/entrar"); // Redireciona para /entrar
            })
            .catch(error => {
                if (error.response.status === 401) {
                    setErrorMessages(["Link expirado, gere outro link na página de login."])
                } else {
                    // A mensagem de texto que vem do back tem erro de concordância, mudamos na mão.
                    if (error.response.data.errors.full_messages[0] === "Senha é muito curto (mínimo: 8 caracteres)")
                        setErrorMessages(["A senha deve possuir no mínimo 8 caracteres."]);
                    else
                        setErrorMessages(error.response.data.errors.full_messages);
                }
            })
    }

    const handleClickShowPassword = () => {
        setShowPassword(!showPassword);
    };

    const handleClickShowPasswordConfirmation = () => {
        setShowPasswordConfirmation(!showPasswordConfirmation);
    };

    return (
        <ThemeProvider theme={theme}>
            <div className="grid grid-cols-1 h-screen">
                <div className="flex flex-col justify-center items-center">
                    <div className="my-10 mx-10">
                        <h1 className="text-3xl max-lg:text-xl font-bold text-turquoise-HC-underline  my-10 text-center">
                            Digite sua nova senha
                        </h1>
                        <h1 className="text-lg max-lg:text-base text-darkGray-HC-white-underline text-balance mb-5 text-center">
                            Assim que sua senha for alterada com sucesso você será redirecionado para o login
                        </h1>
                    </div>
                    <form onSubmit={onSubmit} className="mb-5">
                        <Paper elevation={0} className="w-96 flex flex-col items-center p-5 rounded-xl">
                            <Image
                                src="/mecred.svg"
                                alt="MECRED Logo"
                                width={60}
                                height={24}
                                className="m-5"
                            />
                            {errorMessages && errorMessages.length > 0 ? (
                                <Alert severity="error" className="mb-3">
                                    <ul>
                                        {errorMessages.map((message, index) => (
                                            <li key={index}>{message}</li>
                                        ))}
                                    </ul>
                                </Alert>
                            ) : null}
                            <div className="mb-4 ml-1">
                                <p className="text-darkGray-HC-white-underline text-sm">A senha deve ter no mínimo 8 caracteres.</p>
                            </div>
                            {/* Campo de Nova Senha */}
                            <div className="w-full mb-6">
                            <label htmlFor="senha" className="block text-sm font-medium text-gray-700 mb-1">
                                Nova Senha *
                            </label>
                                <div className="relative">
                                    <input
                                    id="senha"
                                    type={showPassword ? 'text' : 'password'}
                                    className="w-full px-3 py-2 border border-gray-300 rounded-md focus:outline-none focus:ring-2 focus:ring-secondary"
                                    onChange={(e) => setPassword(e.target.value)}
                                    value={password}
                                    alt="Nova Senha"
                                    />
                                    <button
                                    type="button"
                                    onClick={handleClickShowPassword}
                                    className="absolute inset-y-0 right-0 pr-3 flex items-center text-sm leading-5"
                                    >
                                    {showPassword ? <img src="ocultar-senha.svg" alt="Ocultar senha" /> : <img src="ver-senha.svg" alt="Ver senha" />}
                                    </button>
                                </div>
                            </div>

                            {/* Campo de Confirmação de Nova Senha */}
                            <div className="w-full mb-6">
                            <label htmlFor="confirma-senha" className="block text-sm font-medium text-gray-700 mb-1">
                                Confirme Nova Senha *
                            </label>
                                <div className="relative">
                                    <input
                                        id="confirma-senha"
                                        type={showPasswordConfirmation ? 'text' : 'password'}
                                        className="w-full px-3 py-2 border border-gray-300 rounded-md focus:outline-none focus:ring-2 focus:ring-secondary"
                                        onChange={(e) => setPasswordConfirmation(e.target.value)}
                                        value={passwordConfirmation}
                                        alt="Confirme a Nova Senha"
                                    />
                                    <button
                                        type="button"
                                        onClick={handleClickShowPasswordConfirmation}
                                        className="absolute inset-y-0 right-0 pr-3 flex items-center text-sm leading-5"
                                        >
                                        {showPasswordConfirmation ? (
                                            <img src="ocultar-senha.svg" alt="Ocultar senha" />
                                        ) : (
                                            <img src="ver-senha.svg" alt="Ver senha" />
                                        )}
                                    </button>
                                </div>
                            </div>
                            <Button
                                fullWidth
                                disableElevation
                                className="bg-turquoise text-white-HC-dark-underline hover:bg-turquoise-hover"
                                type="submit"
                                alt="Trocar a senha"
                            >
                                Trocar a Senha
                            </Button>
                        </Paper>
                    </form>
                </div>
            </div>
        </ThemeProvider>
    );
}
