import { Checkbox, FormGroup, Paper, TextField, Button } from "@mui/material";
import { FileUpload } from "./FileDrag";
import { useState } from "react";
import FieldLabel from "../../components/FieldLabel";
import Radio from "@mui/material/Radio";
import RadioGroup from "@mui/material/RadioGroup";
import FormControlLabel from "@mui/material/FormControlLabel";
import FormControl from "@mui/material/FormControl";
import FormLabel from "@mui/material/FormLabel";
import SubjectSelection from "./SubjectSelection";
import ObjectTypeSelection from "./ObjectTypeSelection";
import { objectTypes } from "@/app/components/ObjectTypes";
import LicenseSelection from "./LicenseSelection";
import Chip from "@mui/material/Chip";
import TagInput from "./TagsInput";
import AuthorSelection from "./AuthorSelection";

export default function UploadForm({ setStep }) {
  const languagesAvailable = [
    "Português",
    "Inglês",
    "Espanhol",
    "Francês",
    "Alemão",
    "Italiano",
    "Mandarim",
    "Russo",
    "Japonês",
    "LIBRAS",
    "Outro",
  ];

  const scholarityLevel = [
    "Educação Infantil",
    "Ensino Fundamental I (1° até o 5° ano)",
    "Ensino Fundamental II (do 6° até o 9º ano)",
    "Ensino Médio",
    "Ensino Superior",
    "Outros ",
  ];

  const [title, setTitle] = useState("");
  const [description, setDescription] = useState("");
  const [selectedValue, setSelectedValue] = useState("a");
  const [tags, setTags] = useState([]);
  const [author, setAuthor] = useState("");
  const [tagInput, setTagInput] = useState("");
  const [languages, setLanguages] = useState([]);
  const [objectTypes, setObjectTypes] = useState([]);
  const [scholarityLevels, setScholarityLevels] = useState([]);
  const [subjects, setSubjects] = useState([]);
  const [licenseValue, setLicenseValue] = useState(1);

  const handleTitleChange = (e) => setTitle(e.target.value);
  const handleDescriptionChange = (e) => setDescription(e.target.value);
  const handleRadioChange = (e) => setSelectedValue(e.target.value);
  const handleAuthorChange = (e) => setAuthor(e.target.value);
  const handleLicenseValueChange = (e) => setLicenseValue(e.target.value);
  const handleTagInput = (e) => setTagInput(e.target.value);

  const handleCommaInput = (e) => {
    if (e.key === "Enter") {
      if (!tags.includes(tagInput))
        setTags((prevTags) => [...prevTags, tagInput]);
      setTagInput("");
    }
  };

  const handleTagDelete = (tag) => {
    setTags(tags.filter((t) => t !== tag));
  };

  const handleLanguagesChange = (e) => {
    if (languages.includes(e.target.value))
      setLanguages(languages.filter((language) => language !== e.target.value));
    else setLanguages([...languages, e.target.value]);
  };

  const handleObjectsTypeChange = (id) => {
    if (objectTypes.includes(id))
      setObjectTypes(objectTypes.filter((object) => object !== id));
    else setObjectTypes([id]);
  };

  const handleScholarityChange = (e) => {
    if (scholarityLevel.includes(e.target.value))
      setScholarityLevels(
        scholarityLevels.filter((sl) => sl !== e.target.value)
      );
    else setScholarityLevels([...scholarityLevels, e.target.value]);
  };

  const handleSubjectsChange = (id) => {
    if (subjects.includes(id))
      setSubjects(subjects.filter((subject) => subject !== id));
    else setSubjects([...subjects, id]);
  };

  return (
    <form className="mb-10">
      <Paper className="rounded-lg p-10 max-xl:p-5 max-xl:mx-5" elevation={0}>
        <h1 className="text-darkGray-HC-white-underline font-light text-2xl text-center">
          Os campos marcados com * são obrigatórios
        </h1>
        <FieldLabel name="Título do Recurso*" />
        <TextField
          fullWidth
          placeholder="Ex: Jogo Virtual sobre os Biomas do Brasil"
          onChange={handleTitleChange}
          helperText={`${title.length}/100`}
          error={title.length > 100}
          className="mb-5"
        />
        <FieldLabel name="Descrição geral do Recurso*" />
        <TextField
          multiline
          rows={5}
          fullWidth
          placeholder="Quais assuntos são contemplados neste recurso? Para quais objetivos este recurso pode ser destinado?"
          onChange={handleDescriptionChange}
          helperText={`${description.length}/1500`}
          error={description.length > 1500}
          className="mb-5"
        />
        <FieldLabel
          name="Palavras-chave Relacionadas ao Conteúdo do seu Recurso*"
          description="(Use enter para confirmar a palavra-chave)"
        />
        <TextField
          fullWidth
          value={tagInput}
          placeholder="Ex: Bioma, Geografia"
          onChange={handleTagInput}
          onKeyUp={handleCommaInput}
          helperText={`${description.length}/100`}
          className="mb-5"
        />
        <TagInput tags={tags} handleTagDelete={handleTagDelete} />
        <AuthorSelection
          selectedValue={selectedValue}
          handleRadioChange={handleRadioChange}
          author={author}
          handleAuthorChange={handleAuthorChange}
        />
        <ObjectTypeSelection
          selected={objectTypes}
          handleSelection={handleObjectsTypeChange}
        />
        <FieldLabel name="Idiomas do Recurso*" />
        <FormGroup className="flex flex-wrap mb-10" row>
          {languagesAvailable.map((language, index) => (
            <FormControlLabel
              onChange={handleLanguagesChange}
              key={index}
              control={<Checkbox />}
              value={language}
              label={<span className="text-darkGray-HC-white-underline">{language}</span>}
            />
          ))}
        </FormGroup>
        <FieldLabel
          name="Nível de ensino"
          description="(Selecione uma ou mais opções)*"
        />
        <FormGroup className="flex flex-wrap mt-5 mb-10">
          {scholarityLevel.map((sl, index) => (
            <FormControlLabel
              onChange={handleLanguagesChange}
              key={index}
              control={<Checkbox />}
              value={sl}
              label={<span className="text-darkGray-HC-white-underline">{sl}</span>}
            />
          ))}
        </FormGroup>
        <SubjectSelection
          selected={subjects}
          handleSelection={handleSubjectsChange}
        />
        <LicenseSelection
          defaultValue={licenseValue}
          handleRadioChange={handleLicenseValueChange}
        />
      </Paper>
      <div className="flex w-full justify-end p-5">
        <Button
          variant="text"
          className="normal-case w-36 text-base text-darkGray-HC-white-underline font-bold"
        >
          Cancelar
        </Button>
        <Button
          className=" border-secondary w-36 rounded-lg hover:bg-turquoise-hover bg-turquoise text-base text-white-HC-dark-underline font-bold normal-case flex "
          variant="outlined"
          onClick={() => setStep((curr) => curr + 1)}
        >
          Continuar
        </Button>
      </div>
    </form>
  );
}
